# following PEP 440
__version__ = "2.5.1"

# Values starting with such prefix should be handled as external URL.
EXTERNAL_SCHEMES = ('http', 'https', 'ftp', 'ftps', 'sftp', 'webdav', 'webdavs', 'afp', 'smb', 'git', 'svn', 'hg', 'mailto', 'tel')
